unit IOPara;

{FREEWARE 100744,2565}
{Komponente IOPara}
{Parallelschnittstelle als I/O}
{Nur fr Delphi 2.0 !}
{1 ms Timerroutinen aus ct 12 1996}

interface

uses
   Windows, Classes, Messages, Forms, MMSystem;


type
  TPortNumber=(None, LPT1, LPT2, LPT3);
  Bits=1..8;

  TIOPort=class(TComponent)
  private
    FPort:          TPortNumber;
    FPortAddress:   Word;
    PortHandle:     THandle;
    FIOOut:         Byte;
    FIOIn:          Byte;
    FInpChanged:    TNotifyEvent;
    FOutChanged:    TNotifyEvent;
    OldData:        Byte;
    OldStatus:      Byte;
    OldControl:     Byte;
    Data:           Byte;
    Status:         Byte;
    Control:        Byte;
    FTimerID:       Word;
    FWindowHandle:  HWnd;
    FEnabled:       Boolean;
    FInterval:      Word;
    FOnTimer:       TNotifyEvent;
    FPendingEvents: Word;
    procedure SetPort(Value:TPortNumber);
    function GetInput:Byte;
    procedure SetOutput(Value:Byte);
    procedure SetInpChanged(Value:TNotifyEvent);
    procedure SetOutChanged(Value:TNotifyEvent);
    procedure Check(Sender:TObject);
    procedure SetOnTimer(Value:TNotifyEvent);
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    function OpenPort(PortNo:Byte):Boolean;
    function ClosePort:Boolean;
    procedure SetBit(Bitnummer:Bits);
    procedure ResetBit(Bitnummer:Bits);
    function GetBit(Bitnummer:Bits):Boolean;
    procedure SetEnabled(Value:Boolean);
    procedure SetInterval(Value:Word);
    procedure UpdateTimer;
    procedure WndProc(var Msg:TMessage);
  published
    property Port: TPortNumber read FPort write SetPort;
    property BaseAddress: Word read FPortAddress;
    property DataOut: Byte read FIOOut write SetOutput;
    property DataIn: Byte read GetInput write FIOIn;
    property InputChanged: TNotifyEvent read FInpChanged write SetInpChanged;
    property OutputChanged: TNotifyEvent read FOutChanged write SetOutChanged;
    property Enabled: Boolean read fEnabled write SetEnabled;
    property Interval: Word read fInterval write SetInterval;
    property PendingEvents: Word read fPendingEvents;
  end;


procedure Register;

implementation

uses Sysutils;
var IOPortMsg:Word;

procedure TimerProc(wTimerID,msg:Word; dwUser,dw1,dw2:LongInt); export; stdcall;
begin
  with TIOPort(dwUser) do
  begin
    if PendingEvents<5 then
    begin
      PostMessage(fWindowHandle,IOPortMsg,0,0);
      Inc(fPendingEvents);
    end;
  end;
end;

function GetInp(PortAddr:Word):Byte;
assembler; stdcall;
asm
  mov dx,PortAddr
  in al,dx
end;

procedure SetOut(PortAddr:Word; Data:Byte);
assembler; stdcall;
asm
  mov al,Data
  mov dx,PortAddr
  out dx,al
end;

function GetPortAddress(PortNo:Integer):Word;
assembler; stdcall;
asm
  push es
  mov ebx, PortNo
  shl ebx,1
  mov eax,40h
  mov es,ax
  mov ax,es:[ebx+6]
  pop es
end;

constructor TIOPort.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FPort:=None;
  FPortAddress:=0;
  PortHandle:=INVALID_HANDLE_VALUE;
  fWindowHandle:=AllocateHWnd(WndProc);
  SetOnTimer(Self.Check);
  fEnabled:=True;
  fInterval:=1;
end;

destructor TIOPort.Destroy;
begin
  Enabled:=False;
  DeallocateHWnd(fWindowHandle);
  ClosePort;
  inherited Destroy;
end;

procedure TIOPort.WndProc(var Msg:TMessage);
begin
  with Msg do
  begin
    if Msg=IOPortMsg then
    try
      if Enabled then
      begin
        fEnabled:=False;
        dec(fPendingEvents);
        if Assigned(fOnTimer) then fOnTimer(Self);
      end;
    except
      fEnabled:=False;
      Application.HandleException(Self);
    end
    else
      Result:=DefWindowProc(fWindowHandle,Msg,wParam,lParam);
  end;
end;

procedure TIOPort.UpDateTimer;
var MaxInterval:Integer;
begin
  MaxInterval:=Interval;
  if MaxInterval>55 then MaxInterval:=55;
  if Enabled and Assigned(fOnTimer) and (Interval<>0)
    and not (csLoading in ComponentState) then
  begin
     fTimerID:=timeSetEvent(Interval,MaxInterval,TFNTimeCallBack(TimerProc),
                             LongInt(Self),TIME_ONESHOT);
     if fTimerID=0 then
       fTimerID:=timeSetEvent(Interval,0,TFNTimeCallBack(TimerProc),
                             LongInt(Self),TIME_ONESHOT);
  end;
end;

procedure TIOPort.SetEnabled(Value:Boolean);
begin
  if Value<>fEnabled then
  begin
    fEnabled:=Value;
    UpDateTimer;
  end;
end;

procedure TIOPort.SetInterval(Value:Word);
begin
  if Value<>fInterval then
  begin
    fInterval:=Value;
    UpDateTimer;
  end;
end;

function TIOPort.OpenPort(PortNo:Byte):Boolean;
var
  PH: THandle;
  s: string;
begin
  s:='';
  Result:=False;
  ClosePort;
  if not (PortNo in [1..3]) then Exit;
  PH:=CreateFile(PChar('\\.\LPT' + IntToStr(PortNo)),
                       GENERIC_READ or GENERIC_WRITE,
                       0,
                       nil,
                       OPEN_EXISTING,
                       0,
                       0);
  if PH<>INVALID_HANDLE_VALUE then
  begin
    case PortNo of
      1: FPort:=LPT1;
      2: FPort:=LPT2;
      3: FPort:=LPT3;
    end;
    PortHandle:=PH;
    FPortAddress:=GetPortAddress(PortNo);
    SetOut(FPortAddress,0);
    SetOut(FPortAddress+2,4);
    FIOOut:=0;
    FIOIn:=0;
    OldStatus:=0;
    OldControl:=0;
    Status:=0;
    Control:=0;
    Result:=True;
  end;
end;

procedure TIOPort.SetPort(Value:TPortNumber);
begin
  if Value<>FPort then
  begin
    OpenPort(Ord(Value));
    if (csDesigning in ComponentState) and
      (PortHandle<>INVALID_HANDLE_VALUE) then
       CloseHandle(PortHandle);
  end;
end;

function TIOPort.GetInput:Byte;
var StatusReg,ControlReg:Byte;
begin
  if PortHandle <> INVALID_HANDLE_VALUE then
  begin
    StatusReg:=GetInp(FPortAddress+1);
    ControlReg:=GetInp(FPortAddress+2);
    FIOIn:=(StatusReg and $40) shr 6;
    FIOIn:=FIOIn or (((not StatusReg) and $80) shr 6);
    FIOIn:=FIOIn or ((StatusReg and $20) shr 3);
    FIOIn:=FIOIn or ((StatusReg and $10) shr 1);
    FIOIn:=FIOIn or (((not ControlReg) and $02) shl 3);
    FIOIn:=FIOIn or ((StatusReg and $08) shl 2);
    FIOIn:=FIOIn or ((ControlReg and $04) shl 4);
    FIOIn:=FIOIn or (((not ControlReg) and $08) shl 4);
  end
  else
    FIOIn:=0;
  Result:=FIOIn;
end;

procedure TIOPort.SetOutput(Value:Byte);
begin
  if PortHandle <> INVALID_HANDLE_VALUE then
  begin
    SetOut(FPortAddress,Value);
    FIOOut:=Value;
  end else FIOOut:=0;
end;

function TIOPort.ClosePort:Boolean;
begin
  if PortHandle <> INVALID_HANDLE_VALUE then
  begin
    SetOut(FPortAddress,0);
    SetOut(FPortAddress+2,(GetInp(FPortAddress+2) and $F0) or $0C);
    Result:=CloseHandle(PortHandle);
  end
  else
    Result := False;
  PortHandle:=INVALID_HANDLE_VALUE;
  FPort:=None;
  FPortAddress:=0;
end;

procedure TIOPort.SetInpChanged(Value:TNotifyEvent);
begin
  FInpChanged:=Value;
  UpDateTimer;
end;

procedure TIOPort.SetOutChanged(Value:TNotifyEvent);
begin
  FOutChanged:=Value;
  UpDateTimer;
end;

procedure TIOPort.SetBit(Bitnummer:Bits);
begin
  FIOOut:=FIOOut or (1 shl pred(Bitnummer));
  SetOutput(FIOOut);
end;

procedure TIOPort.ResetBit(Bitnummer:Bits);
begin
  FIOOut:=FIOOut and not (1 shl pred(Bitnummer));
  SetOutput(FIOOut);
end;

function TIOPort.GetBit(Bitnummer:Bits):Boolean;
begin
  GetBit:=GetInput and (1 shl pred(Bitnummer))>0;
end;

procedure TIOPort.SetOnTimer(Value:TNotifyEvent);
begin
  fOnTimer:=Value;
  UpDateTimer;
end;

procedure TIOPort.Check(Sender:TObject);
begin
  Enabled:=False;
  if PortHandle <> INVALID_HANDLE_VALUE then
  begin
    Data:=GetInp(FPortAddress);
    Status:=GetInp(FPortAddress+1);
    Control:=GetInp(FPortAddress+2);
    if (Control<>OldControl) or (Status<>OldStatus) then
    begin
      OldControl:=Control;
      OldStatus:=Status;
      FIOIn:=GetInput;
      if Assigned(fInpChanged) and not (csLoading in ComponentState) then
         FInpChanged(Sender);
    end;
    if (Data<>OldData) then
    begin
      OldData:=Data;
      if Assigned(fOutChanged) and not (csLoading in ComponentState) then
         FOutChanged(Sender);
    end;
    Enabled:=True;
  end;
end;


procedure Register;
begin
  RegisterComponents('Beispiele', [TIOPort]);
end;

initialization
  IOPortMsg:=RegisterWindowMessage('TIOPort');
end.


